from bottle import route, run, template, TEMPLATE_PATH,request, static_file
import os 
import time
import re
import subprocess
import json

def exec2(cmd: str):
    return subprocess.getstatusoutput(cmd)

def check_iface(iface: str) -> bool:
    if iface == "lo" or iface.startswith("can"):
        return False
    return True

def pick_ip(txt: str) -> str:
    return re.findall(r'\b(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9][0-9]|[0-9])(?:\.(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9][0-9]|[0-9])){3}\b', txt)

class NetTool:
    def __init__(self):
        (state, _) = exec2("systemctl is-active NetworkManager")
        self._support =  state == 0
        self._nets = {}
        self.load()
    
    def is_support(self) -> bool:
        return self._support
    
    def is_auto(self, connection: str) -> bool:
        (state, msg) = exec2(f"nmcli connection show '{connection}' | grep ipv4.method | grep 'manual'")
        if state == 0:
            return False
        return True

    def load(self):
        if not self._support:
            return None
        self._nets = {}
        exculde_iface = "lo"
        for iface in os.listdir("/sys/class/net/"):
            if not check_iface(iface):
                continue
            data = {}
            (state, msg) = exec2(f"nmcli device show {iface}")
            if state == 0:
                for line in msg.splitlines():
                    if line .startswith("GENERAL.TYPE:"):
                        data["wifi"] = " wifi" in line
                    elif line.startswith("GENERAL.STATE:"):
                        data["connected"] = "(connected)" in line
                    elif line.startswith("IP4.ADDRESS"):
                        data["ip"] = pick_ip(line)
                    elif line.startswith("IP4.GATEWAY:"):
                        data["gw"] = pick_ip(line)    
                    elif line.startswith("GENERAL.CONNECTION:"):
                        data["connection"] = line[20:].strip()
                if "connected" in data and "connection" in data and data["connected"]:   
                    data["auto"] = self.is_auto(data["connection"])

            self._nets[iface] = data
    
    def nets(self):
        self.load()
        return json.dumps(self._nets)
    
    def set_ip(self, connection: str, ip: str, gw: str) -> bool:
        if not self._support:
            return False
        if ip == "0.0.0.0":
            (state, msg) = exec2(f"nmcli connection modify '{connection}' ipv4.method auto")
            # print(f"[{state}] {msg}")
            return state == 0
        print(f"nmcli con mod '{connection}' ipv4.addresses {ip}/24 ipv4.gateway {gw} ipv4.method manual")
        (state, msg) = exec2(f"nmcli con mod '{connection}' ipv4.addresses {ip}/24 ipv4.gateway {gw} ipv4.method manual")
        # print(f"[{state}] {msg}")
        os.system("systemctl restart NetworkManager.service")
        return state == 0

G_NET_TOOL = NetTool()

def load_i2c_gpio():
    gpiochip={}
    name = None
    data = None
    with open('/sys/kernel/debug/gpio', 'r') as file:
        lines = file.readlines()
        for line in lines:
            if line.startswith(" gpio-"):
               if data is not None:
                   items = re.findall("^ gpio-(\d+) \((\S+).*\)", line)
                   if len(items) == 1:
                       
                       data[items[0][1]]=items[0][0]
            else:
                if data is not None and name is not None:
                    gpiochip[name] = data
                name = None
                data = None
                if line.startswith("gpiochip"):
                    if "parent: i2c/" in line:
                        line=line.strip()
                        names = re.findall(r"^gpiochip(\d+):", line)
                        if len(names) == 1:
                            name = names[0].strip()
                            data = {} 
    if data is not None and name is not None:
        gpiochip[name] = data
        name = None
        data = None
    return gpiochip

def static_path():
    abs_app_dir_path = os.path.dirname(os.path.realpath(__file__)) + os.path.sep+ "..";
    abs_views_path = os.path.join(abs_app_dir_path, 'static');
    return abs_views_path;

def exec1(cmd):
    with os.popen(cmd, "r") as f:
        return f.read()

def search_str(pattern, data):
    return re.search( pattern, data, re.M|re.I)

def load_info():
    t_deb_str = exec1("dpkg -l | grep ed- | awk '{print $2}'");
    t_debs = t_deb_str.splitlines();
    t_info = {}
    for t_deb in t_debs:
        if t_deb == "ed-codesys-pem":
            continue
        elif t_deb.startswith("ed-codesys-"):
            t_power = t_deb[11:]
            t_info["power"] = t_power.strip()
        else:
            b_firmware = search_str(r'^ed-(.*)-firmware$', t_deb)
            if b_firmware:
                t_info["firmware"] = b_firmware.group(1).strip()
                t_info["firmware"] = t_info["firmware"] + "(" + exec1("dpkg -l | grep ed-"+t_info["firmware"]+"-firmware | awk '{print $3}'").strip() +")"
            else:
                b_bsp = search_str(r'^ed-(.*)-bsp$', t_deb)
                if b_bsp:
                    t_info["firmware"] = b_bsp.group(1).strip()
                    t_info["firmware"] = t_info["firmware"] + "(" + exec1("dpkg -l | grep ed-"+t_info["firmware"]+"-bsp | awk '{print $3}'").strip() +")"
    return t_info
        

@route('/static/<filename:path>')
def static(filename):
    return static_file(filename, root=static_path());

@route('/backup', method='POST')
def do_backup():
    t_time = time.strftime("%Y%m%d", time.localtime());
    t_static = static_path();
    t_output = "{}/downloads/EDA_pem_backup_{}.zip".format(t_static, t_time);
    os.system("rm -f {}".format(t_output));
    if os.system("zip {} /etc/eda/* -x '/etc/eda/3S.dat'".format(t_output)) == 0 :
        return "/static/downloads/EDA_pem_backup_{}.zip".format(t_time);
    return str("-1");

@route('/create', method='POST')
def do_backup():
    t_static = static_path();
    if os.system("mkdir -p /etc/eda && cd /etc/eda/ && eda-codesys") == 0:
        os.system("cp /etc/eda/eda-rsa.pub {}/downloads/eda-rsa-v2.pub".format(t_static));
        return "/static/downloads/eda-rsa-v2.pub";
    return str("-2");

@route('/device/file', method='POST')
def do_device_file():
    t_static = static_path();
    t_exist = False
    if os.path.exists("/etc/eda"):
        for f in os.listdir("/etc/eda"):
            if f.endswith(".xml"):
                t_exist = True;
                break
    if t_exist:
        # os.system(f"cp /etc/eda/{t_file} {t_static}/downloads/{t_file}")
        # return f"/static/downloads/{t_file}"
        t_output = "{}/downloads/Devdesc.zip".format(static_path());
        if os.system("zip -j {} /etc/eda/*.xml".format(t_output)) == 0 :
            return "/static/downloads/Devdesc.zip";
    return str("device_file_err1");

@route('/close', method='POST')
def do_close():
    os.system("systemctl disable eda-pem-manager.service")
    os.system("systemctl stop eda-pem-manager.service")
    return str("-2");

@route('/license', method='POST')
def do_license():
    # name = request.forms.name
    data = request.files.data
    if data and data.file:
        # raw = data.file.read() # This is dangerous for big files
        filename = data.filename
        # filepath = os.path.join("/etc/eda/", filename);
        filepath = "/etc/eda/eda-v1.dat"
        data.save(filepath, overwrite=True);
        # os.remove(filepath);
        os.system("systemctl restart codesyscontrol.service");
        return "You uploaded %s." % (filename)
    return "You missed a field."

@route('/upload', method='POST')
def do_upload():
    # name = request.forms.name
    data = request.files.data
    if data and data.file:
        # raw = data.file.read() # This is dangerous for big files
        filename = data.filename
        filepath = os.path.join(static_path(), filename);
        data.save(filepath, overwrite=True);
        os.system("mkdir -p /etc/eda && unzip -o {} -d /".format(filepath));
        os.remove(filepath);
        os.system("systemctl restart codesyscontrol.service");
        return "You uploaded %s." % (filename)
    return "You missed a field."

@route('/info/set/ip', method='POST')
def net_ip_set():
    data = request.json
    print(data)
    if "connection" in data and "ip" in data and "gw" in data:
        if G_NET_TOOL.set_ip(data["connection"], data["ip"], data["gw"]):
            return "1"
    return "0"

@route('/net/info')
def net_info():
    return G_NET_TOOL.nets()

@route('/admin')
def index():
    return template('index', name="admin", info=load_info(), gpiochips=load_i2c_gpio(), nets=G_NET_TOOL.nets());

@route('/')
def index():
    return template('index', name="user", info=load_info(), gpiochips=load_i2c_gpio(), nets=G_NET_TOOL.nets());

if __name__ == '__main__':
    abs_app_dir_path = os.path.dirname(os.path.realpath(__file__)) + os.path.sep+ "..";
    abs_views_path = os.path.join(abs_app_dir_path, 'views');
    abs_pems_path = os.path.join(abs_app_dir_path, 'pems');
    TEMPLATE_PATH.insert(0,abs_views_path);
    os.system("mkdir -p {}/downloads".format(static_path()))
    run(host='0.0.0.0', port=8100, debug=False);
    # run(host='0.0.0.0', port=8080, debug=False,server='gunicorn',reloader=1,keyfile='{}/key.pem'.format(abs_pems_path),certfile='{}/cert.pem'.format(abs_pems_path));